#!/usr/bin/perl -w

use CGI qw/:standard *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

$dim = param('dim');
unless ($dim) {
    $dim = 3;
    param('dim', $dim);
}

print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'initial procurement problem'),
    h1('initial procurement problem'), ln;


sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( $e and $e =~ m/([-+]?\d+)/
	 and $min <= int($1) and int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i, $j);

    set_param('penalty', 1, 999);
    for ($i=0; $i<$dim; $i++) {
	set_param("cost:$i", 1, 99);
	set_param("rate:$i", 1, 99);
    }
}

sub print_html_table {
    init_param();
    print <<'EOF';
<pre>
We consider the initial procurement problem for a reparable inventory system.
We assume that the steady state number of items 
in the replenishment cycle has a Poisson distribution.

n items (i = 1, 2, ... ,n)
</pre>
EOF
    print start_form, hidden('dim'), start_div({-align=>'center'}), ln;

    @index = (1..10);
    print p, 'number of items = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change');
    print p, 'penalty = ', textfield(-name=>'penalty', -size=>4, -default=>param('penalty')), ln;

    print start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ '', 1 .. $dim ])]);
    foreach $e ('rate', 'cost') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }
    print end_table, ln;
    print p, "(Note: An inappropriate value will be replaced with a random number.)\n";
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_div, end_form, ln;


    print <<'EOF';
<pre>
rate (<img>): The steady state Poisson demand rates, 0< rate
cost (<img>): The spare purchases cost of a unit of item i
penalty (<img>): The penalty of a backorder

(<img>)f(x) = ... graphic image by TeX

The first term is the backorder penalty which is computed by
the product of the cost of a backorder by
 the steady state expected number of backorders of the random item
with the most backorders.
The second term is the cost of spares purchases.
</pre>
EOF
}

sub change_dim {
    $dim = param('newdim');
    param('dim', $dim);
}

sub call_executable {
    print '<pre>', ln;
    open(OUT, "|./odicon D $dim Z") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    print OUT param('penalty'), ln;
    foreach $e ('cost', 'rate') {
	for ($j=0; $j<$dim; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    close OUT;
    print '</pre>', ln;
}


if (param('Minimize')) {
    print_html_table();
    call_executable();
} elsif (param('Change')) {
    change_dim();
    print_html_table();
} else {
    print_html_table();
}

print end_html, ln;
